package com.zd.chemical.mapper;

import java.util.List;
import com.zd.chemical.domain.HxpCabinetlockLog;
import com.zd.chemical.domain.vo.HxpCabinetlockLogSearch;
import com.zd.chemical.domain.vo.HxpCabinetlockLogVo;
import org.apache.ibatis.annotations.Param;

/**
 * 【请填写功能名称】Mapper接口
 *
 * @author zd
 * @date 2022-07-11
 */
public interface HxpCabinetlockLogMapper
{
    /**
     * 查询【请填写功能名称】
     *
     * @param id 【请填写功能名称】主键
     * @return 【请填写功能名称】
     */
    HxpCabinetlockLog selectHxpCabinetlockLogById(Long id);

    /**
     * 查询【请填写功能名称】列表
     *
     * @param hxpCabinetlockLogSearch 【请填写功能名称】
     * @return 【请填写功能名称】集合
     */
    List<HxpCabinetlockLogVo> selectHxpCabinetlockLogList(HxpCabinetlockLogSearch hxpCabinetlockLogSearch);

    /**
     * 根据主键集合查询【请填写功能名称】列表
     *
     * @param ids 主键集合
     * @return 【请填写功能名称】集合
     */
    List<HxpCabinetlockLog> getListByIds(List <Long> ids);

    /**
     * 新增【请填写功能名称】
     *
     * @param hxpCabinetlockLog 【请填写功能名称】
     * @return 结果
     */
    int insertHxpCabinetlockLog(HxpCabinetlockLog hxpCabinetlockLog);

    /**
     * 修改【请填写功能名称】
     *
     * @param hxpCabinetlockLog 【请填写功能名称】
     * @return 结果
     */
    int updateHxpCabinetlockLog(HxpCabinetlockLog hxpCabinetlockLog);

    /**
     * 删除【请填写功能名称】
     *
     * @param id 【请填写功能名称】主键
     * @return 结果
     */
    int deleteHxpCabinetlockLogById(Long id);

    /**
     * 批量删除【请填写功能名称】
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    int deleteHxpCabinetlockLogByIds(Long[] ids);

    HxpCabinetlockLog selectByLockId(@Param("subId") Long subId, @Param("lockId") Integer lockId);
}
